/****************************************************************************
**
** Copyright (C) 2023 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt Design Viewer of the Qt Toolkit.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#ifndef DSCONNECTOR_H
#define DSCONNECTOR_H

#include <QObject>
#include <QTcpServer>
#include <QTcpSocket>
#include <QTimer>
#include <QUdpSocket>

class DesignStudioConnector : public QObject
{
    Q_OBJECT
public:
    explicit DesignStudioConnector(const quint16 port = 40000, QObject *parent = nullptr);

public slots:
    void sendProjectReceived();

private:
    // Tcp connection members
    QTcpServer m_tcpServer;
    QScopedPointer<QTcpSocket> m_tcpSocket;
    const quint32 m_tcpPort;

    // Udp connection members
    QTimer m_ipUpdateTimer;
    QString m_ipv4Addr;

    // Other members
    QByteArray m_projectData;
    bool m_receivingData;

    // Member functions
    void initTcpServer();
    void receiveProject();
    void updateIpv4Addr();
    void clientConnected();

signals:
    void networkStatusUpdated(QString);
    void projectReceived(QByteArray);
    void projectIncoming();
};

#endif // DSCONNECTOR_H
