import QtQuick
import QtQuick.Controls 6.4
import QtQuick.Layouts


Item {
    id: homePage

    ColumnLayout {
        anchors.fill: parent

        GridLayout{
            id: gridLayout

            Item {
                id: item2
                Layout.preferredWidth: 10
                Layout.preferredHeight: 3
                Layout.fillWidth: true
                Layout.fillHeight: true
            }

            Text {
                id: qrCodeStatus
                text: {
                    if (backend.userHash() === '') {
                        return qsTr("No user is registered. To access your shared projects:\n\n1. Open the project with Design Studio (4.4 or later).\n2. Select File -> Share Application Online.\n3. Scan the QR code.")
                    }

                    return qsTr("User registration is completed.\nScan a new QR code to access the shared project from a different user.");
                }
                font.pixelSize: 12
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                Layout.fillWidth: true
                Connections {
                    target: backend
                    function onUserHashChanged(){
                        qrCodeStatus.text = qsTr("User registration is completed.\nScan a new QR code to access the shared project from a different user.");
                    }
                }
            }

            Button {
                id: scanQrCode
                text: qsTr("Scan QR code")
                onClicked: backend.scanQrCode()
                Layout.fillWidth: true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            }

            Rectangle {
                id: sep1
                height: 2
                color: 'black'
                Layout.fillWidth: true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            }

            Rectangle {
                id: sep2
                height: 2
                color: 'black'
                Layout.fillWidth: true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            }

            Item {
                id: item4
                Layout.preferredWidth: 10
                Layout.preferredHeight: 10
                Layout.fillWidth: true
                Layout.fillHeight: true
            }

            Text {
                id: dsInstructions
                text: "Create a connection to  Qt Design Studio:
    Goto File > Deploy Project to Android, and type in the following IP address:"
                font.pixelSize: 12
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                Layout.fillWidth: true
            }

            Text {
                id: ipAdress
                text: "Waiting for backend to be initialized..."
                font.pixelSize: 16
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                Layout.fillWidth: true
                Connections {
                    target: backend
                    function onNetworkUpdated(newStatus){
                        ipAdress.text = newStatus
                    }
                }
            }

            Item {
                id: item5
                Layout.preferredWidth: 10
                Layout.preferredHeight: 10
                Layout.fillWidth: true
                Layout.fillHeight: true
            }
        }

        Item {
            id: item6
            Layout.preferredWidth: 10
            Layout.preferredHeight: 10
            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        ColumnLayout {
            id: column2
            Layout.fillWidth: true

            ComboBox {
                id: projectList
                Layout.fillWidth: true
                enabled: true
                textRole: "appName"
                model: backend.projectList
                onModelChanged: {
                    if (backend.projectList.length > 0) {
                        projectList.displayText = backend.projectList[0].appName;
                        projectList.enabled = true;
                        downloadUserProject.enabled = true;
                    } else {
                        projectList.displayText = qsTr("No project is available");
                        projectList.enabled = false;
                        downloadUserProject.enabled = false;
                    }
                }
                onCurrentIndexChanged: {
                    displayText = model[currentIndex].appName;
                }
                displayText: qsTr("Scan QR code to access your projects");
                currentIndex: -1
                Layout.preferredHeight: 50
            }

            TextField {
                id: password
                property bool showText: false
                Layout.fillWidth: true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
                placeholderText: qsTr("Project password")
                echoMode: showText ? TextInput.Normal : TextInput.Password


                MouseArea {
                    id: mouseArea
                    anchors.right: parent.right
                    height: parent.height
                    width: height

                    Image {
                        id: eyeIcon
                        anchors.fill: parent
                        anchors.margins: 15
                        anchors.leftMargin: 10
                        anchors.rightMargin: 10
                        source: "content/images/closed_eye.png"
                    }

                    onClicked: {
                        if(password.showText) {
                            eyeIcon.source = "content/images/closed_eye.png"
                        } else {
                            eyeIcon.source = "content/images/open_eye.png"
                        }

                        password.showText = !password.showText;
                    }
                }
            }

            Button {
                id: downloadUserProject
                text: qsTr("Run Project")
                onClicked: {
                    backend.runUserProject(projectList.displayText, password.text);
                }
                enabled: false
                Layout.fillWidth: true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            }
        }

        Item {
            id: item3
            Layout.fillHeight: true
            Layout.fillWidth: true
            Layout.preferredHeight: 10
            Layout.preferredWidth: 10
        }

        ColumnLayout {
            id: column
            Layout.fillWidth: true
            visible: false

            Text {
                id: downloadInstructions
                text: qsTr("Enter project URL and then click the button below")
                font.pixelSize: 12
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                Layout.fillWidth: true
            }

            TextField {
                id: urlTextField
                horizontalAlignment: Text.AlignHCenter
                placeholderText: qsTr("Enter URL")
                Layout.fillWidth: true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            }

            RowLayout {
                id: rowLayout
                Button {
                    id: downloadButton
                    text: qsTr("Download and Run Project")
                    onClicked: backend.runOnlineProject(urlTextField.text)
                    enabled: urlTextField.text !== ''
                    Layout.fillWidth: true
                    Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
                }
            }

            Connections {
                target: backend
                function onUrlUpdated(newUrl){
                    urlTextField.text = newUrl;
                }
            }
        }
    }
    states: [
        State {
            name: "State1"
            when: homePage.height >= 400

            PropertyChanges {
                target: gridLayout
                columns: 1
            }

            PropertyChanges {
                target: sep1
                visible: false
            }

            PropertyChanges {
                target: sep2
                visible: false
            }

            PropertyChanges {
                target: item5
                visible: true
            }
        },
        State {
            name: "State2"
            when: homePage.height <= 400

            PropertyChanges {
                target: item2
                visible: false
            }

            PropertyChanges {
                target: item4
                visible: false
            }

            PropertyChanges {
                target: sep1
                visible: true
            }

            PropertyChanges {
                target: sep2
                visible: true
            }

            PropertyChanges {
                target: item5
                visible: false
            }

            PropertyChanges {
                target: gridLayout
                columns: 2
            }

        }
    ]
}
